module interfcs

INTERFACE
	PURE FUNCTION MatVar(a,d)
		USE nrtype
		IMPLICIT NONE
		INTEGER(I4B), INTENT(IN) :: d
		REAL(DP) :: MatVar(1:d,1:d)
		REAL(DP), INTENT(IN) :: a(:)
	END FUNCTION	

	SUBROUTINE NDer(N,d,ND) 
		USE nrtype
		INTEGER(I4B), INTENT(IN) :: d, N  
		REAL(DP), INTENT(OUT) :: ND(:,:)			
	END SUBROUTINE
END INTERFACE


INTERFACE
   	SUBROUTINE  bandedLUinv(a,N,m1,m2,aInv)
		USE nrtype
		IMPLICIT NONE
		REAL(DP), INTENT(IN) :: a(:,:)				
		INTEGER(I4B), INTENT(IN) :: N, m1,m2
		REAL(DP), INTENT(OUT) :: aInv(:,:)				
   	END SUBROUTINE 
END INTERFACE

INTERFACE
   	SUBROUTINE LUinv(a)
		USE nrtype
		IMPLICIT NONE
		REAL(DP), INTENT(INOUT) :: a(:,:)
   	END SUBROUTINE
END INTERFACE

INTERFACE
	SUBROUTINE derivs(t,y,dydt)
		USE nrtype
		IMPLICIT NONE
		REAL(DP), INTENT(IN) :: t
		REAL(DP), DIMENSION(:), INTENT(IN) :: y
		REAL(DP), DIMENSION(:), INTENT(OUT) :: dydt
	END SUBROUTINE
END INTERFACE

INTERFACE
	SUBROUTINE banbks(a,m1,m2,al,indx,b)
	USE nrtype
	INTEGER(I4B), INTENT(IN) :: m1,m2
	INTEGER(I4B), DIMENSION(:), INTENT(IN) :: indx
	REAL(DP), DIMENSION(:,:), INTENT(IN) :: a,al
	REAL(DP), DIMENSION(:), INTENT(INOUT) :: b
	END SUBROUTINE banbks
END INTERFACE

INTERFACE
	SUBROUTINE bandec(a,m1,m2,al,indx,d)
	USE nrtype
	INTEGER(I4B), INTENT(IN) :: m1,m2
	INTEGER(I4B), DIMENSION(:), INTENT(OUT) :: indx
	REAL(DP), INTENT(OUT) :: d
	REAL(DP), DIMENSION(:,:), INTENT(INOUT) :: a
	REAL(DP), DIMENSION(:,:), INTENT(OUT) :: al
	END SUBROUTINE bandec
END INTERFACE

INTERFACE
	SUBROUTINE banmul(a,m1,m2,x,b)
	USE nrtype
	INTEGER(I4B), INTENT(IN) :: m1,m2
	REAL(DP), DIMENSION(:), INTENT(IN) :: x
	REAL(DP), DIMENSION(:), INTENT(OUT) :: b
	REAL(DP), DIMENSION(:,:), INTENT(IN) :: a
	END SUBROUTINE banmul
END INTERFACE

INTERFACE
	SUBROUTINE lubksb(a,indx,b)
	USE nrtype
	REAL(DP), DIMENSION(:,:), INTENT(IN) :: a
	INTEGER(I4B), DIMENSION(:), INTENT(IN) :: indx
	REAL(DP), DIMENSION(:), INTENT(INOUT) :: b
	END SUBROUTINE lubksb
END INTERFACE

INTERFACE
	SUBROUTINE ludcmp(a,indx,d)
	USE nrtype
	REAL(DP), DIMENSION(:,:), INTENT(INOUT) :: a
	INTEGER(I4B), DIMENSION(:), INTENT(OUT) :: indx
	REAL(DP), INTENT(OUT) :: d
	END SUBROUTINE ludcmp
END INTERFACE

INTERFACE
	SUBROUTINE setRHS(N,d,m1,m2,av,lambda,U,V,ch,b)
		USE nrtype
		IMPLICIT NONE
		INTEGER(I4B), INTENT(IN) :: N,d,m1,m2
		REAL(DP), INTENT(IN)	:: av(:), lambda, ch
		REAL(DP), INTENT(OUT)	::  U(:,:), V(:,:), b(:)
	END SUBROUTINE
END INTERFACE



INTERFACE
	FUNCTION FDM(N,d,m1,m2,ch)
		USE nrtype
		IMPLICIT NONE
		INTEGER(I4B), INTENT(IN):: N,m1,m2,d
		REAL(DP), INTENT(IN) :: ch
		REAL(DP) :: FDM(N*d+1,m1+1+m2)
	END FUNCTION
END INTERFACE

INTERFACE
	SUBROUTINE Woodbury(LHM,U,V,N,d,m1,m2,b,al,indx,Ho,Hinv,deltaA)
		USE nrtype
		IMPLICIT NONE
		REAL(DP), INTENT(INOUT) :: LHM(:,:), U(:,:)
		REAL(DP), INTENT(IN) :: V(:,:)
		INTEGER(I4B), INTENT(IN) :: N,m1,m2,d
		REAL(DP), INTENT(IN) :: al(:,:)
		INTEGER(I4B), INTENT(IN) :: indx(:)
		REAL(DP), INTENT(IN) :: b(:) 
		REAL(DP), INTENT(OUT) :: Ho(:,:)
		REAL(DP), INTENT(OUT) :: Hinv(:,:)
		REAL(DP), INTENT(OUT) :: deltaA(:)
	END SUBROUTINE
END INTERFACE


INTERFACE
	FUNCTION UnitMatrix(N)
		USE nrtype
		IMPLICIT NONE
		INTEGER(I4B) :: N
		REAL(DP) :: UnitMatrix(N,N)
	END FUNCTION
END INTERFACE


INTERFACE
	SUBROUTINE setLHM(N,d,m1,m2,av,lambda,ch,LHM)
		USE nrtype
		IMPLICIT NONE
		INTEGER(I4B), INTENT(IN) :: N,d,m1,m2
		REAL(DP), INTENT(IN) :: av(:), lambda, ch 
		REAL(DP), INTENT(OUT) :: LHM(:,:)
	END SUBROUTINE
END INTERFACE


INTERFACE
	FUNCTION vector_abs_max(b)
		USE nrtype
		IMPLICIT NONE
		REAL(DP) :: vector_abs_max
		REAL(DP), INTENT(IN) :: b(:)
	END FUNCTION
END INTERFACE

INTERFACE
	SUBROUTINE setUV(N,d,ch,U,V)
		USE nrtype
		IMPLICIT NONE
		INTEGER(I4B), INTENT(IN) :: N,d
		REAL(DP), INTENT(IN) :: ch
		REAL(DP), INTENT(OUT) :: U(:,:),V(:,:) 
	END SUBROUTINE
END INTERFACE

INTERFACE
	SUBROUTINE setSmallUV(N,d,Uv,Vv)
		USE nrtype
		IMPLICIT NONE
		INTEGER(DP), INTENT(IN) :: N,d
		REAL(DP), INTENT(OUT)	:: Uv(:), Vv(:)
	END SUBROUTINE
END INTERFACE

INTERFACE
	SUBROUTINE instantWoodbury(Usaved,V,Hinv,b,x)
		USE nrtype
		IMPLICIT NONE
		REAL(DP), INTENT(INOUT) ::  Usaved(:,:)
		REAL(DP), INTENT(IN) :: V(:,:)
		REAL(DP), INTENT(INOUT) :: Hinv(:,:)
		REAL(DP), INTENT(IN) :: b(:)
		REAL(DP), INTENT(OUT) :: x(:)
	END SUBROUTINE
END INTERFACE



INTERFACE
	SUBROUTINE fastWoodbury(LHM,Usaved,V,Hinv,N,d,m1,m2,b,al,indx,x)
		USE nrtype
		IMPLICIT NONE
		REAL(DP), INTENT(IN) :: LHM(:,:)
		REAL(DP), INTENT(INOUT) ::  Usaved(:,:)
		REAL(DP), INTENT(IN) :: V(:,:)
		REAL(DP), INTENT(IN) :: Hinv(:,:)
		INTEGER(I4B), INTENT(IN) :: N,m1,m2,d
		REAL(DP), INTENT(IN) :: al(:,:)
		INTEGER(I4B), INTENT(IN) :: indx(:)
		REAL(DP), INTENT(IN) :: b(:)
		REAL(DP), INTENT(OUT) :: x(:)
	END SUBROUTINE
END INTERFACE


INTERFACE
	SUBROUTINE StoredWoodbury(aInv,U,V)
		USE nrtype
		IMPLICIT NONE
		REAL(DP), INTENT(INOUT) :: aInv(:,:)
		REAL(DP), INTENT(IN) :: U(:,:), V(:,:)
	END SUBROUTINE
END INTERFACE

!INTERFACE
!	END SUBROUTINE
!END INTERFACE
!INTERFACE
!	END SUBROUTINE
!END INTERFACE
!INTERFACE
!	END SUBROUTINE
!END INTERFACE
!INTERFACE
!	END SUBROUTINE
!END INTERFACE
!INTERFACE
!	END SUBROUTINE
!END INTERFACE
!INTERFACE
!	END SUBROUTINE
!END INTERFACE
!INTERFACE
!	END SUBROUTINE
!END INTERFACE
!INTERFACE
!	END SUBROUTINE
!END INTERFACE


end module

